@@ -97,7 +97,11 @@ Escapes:
 `\H`
 : not hex digit (`[^0-9A-Fa-f]`) \
 `\e`
-: escape control character (`\x1B`)
+: escape control character (`\x1B`) \
+`\K`
+: keep text matched so far out of the overall match ([docs](https://www.regular-expressions.info/keep.html))\
+`\G`
+: anchor to where the previous match ended ([docs](https://www.regular-expressions.info/continue.html))
 
 Backreferences:
 
@@ -138,10 +142,21 @@ assert!(re.is_match("abcc").unwrap());
 assert!(!re.is_match("abc").unwrap());
 ```
 
+Conditionals - if/then/else:
+
+`(?(1))`
+: continue only if first capture group matched \
+`(?(<name>))`
+: continue only if capture group named *name* matched \
+`(?(1)true_branch|false_branch)`
+: if the first capture group matched then execute the true_branch regex expression, else execute false_branch ([docs](https://www.regular-expressions.info/conditional.html)) \
+`(?(condition)true_branch|false_branch)`
+: if the condition matches then execute the true_branch regex expression, else execute false_branch from the point just before the condition was evaluated
+
 [regex]: https://crates.io/crates/regex
 */
 
-#![doc(html_root_url = "https://docs.rs/fancy-regex/0.7.1")]
+#![doc(html_root_url = "https://docs.rs/fancy-regex/0.11.0")]
 #![deny(missing_docs)]
 #![deny(missing_debug_implementations)]
 
@@ -163,9 +178,9 @@ mod vm;
 use crate::analyze::analyze;
 use crate::compile::compile;
 use crate::parse::{ExprTree, NamedGroups, Parser};
-use crate::vm::Prog;
+use crate::vm::{Prog, OPTION_SKIPPED_EMPTY_MATCH};
 
-pub use crate::error::{Error, Result};
+pub use crate::error::{CompileError, Error, ParseError, Result, RuntimeError};
 pub use crate::expand::Expander;
 pub use crate::replacer::{NoExpand, Replacer, ReplacerRef};
 use std::borrow::Cow;
@@ -245,11 +260,24 @@ impl<'r, 't> Iterator for Matches<'r, 't> {
             return None;
         }
 
-        let mat = match self.re.find_from_pos(self.text, self.last_end) {
-            Err(error) => return Some(Err(error)),
-            Ok(None) => return None,
-            Ok(Some(mat)) => mat,
+        let option_flags = if let Some(last_match) = self.last_match {
+            if self.last_end > last_match {
+                OPTION_SKIPPED_EMPTY_MATCH
+            } else {
+                0
+            }
+        } else {
+            0
         };
+        let mat =
+            match self
+                .re
+                .find_from_pos_with_option_flags(self.text, self.last_end, option_flags)
+            {
+                Err(error) => return Some(Err(error)),
+                Ok(None) => return None,
+                Ok(Some(mat)) => mat,
+            };
 
         if mat.start == mat.end {
             // This is an empty match. To ensure we make progress, start
@@ -608,12 +636,21 @@ impl Regex {
     /// Note that in some cases this is not the same as using the `find`
     /// method and passing a slice of the string, see [Regex::captures_from_pos()] for details.
     pub fn find_from_pos<'t>(&self, text: &'t str, pos: usize) -> Result<Option<Match<'t>>> {
+        self.find_from_pos_with_option_flags(text, pos, 0)
+    }
+
+    fn find_from_pos_with_option_flags<'t>(
+        &self,
+        text: &'t str,
+        pos: usize,
+        option_flags: u32,
+    ) -> Result<Option<Match<'t>>> {
         match &self.inner {
             RegexImpl::Wrap { inner, .. } => Ok(inner
                 .find_at(text, pos)
                 .map(|m| Match::new(text, m.start(), m.end()))),
             RegexImpl::Fancy { prog, options, .. } => {
-                let result = vm::run(prog, text, pos, 0, options)?;
+                let result = vm::run(prog, text, pos, option_flags, options)?;
                 Ok(result.map(|saves| Match::new(text, saves[0], saves[1])))
             }
         }
@@ -1181,11 +1218,24 @@ pub enum Expr {
     /// Back reference to a capture group, e.g. `\1` in `(abc|def)\1` references the captured group
     /// and the whole regex matches either `abcabc` or `defdef`.
     Backref(usize),
-    /// Back reference to a named capture group.
-    NamedBackref(String),
     /// Atomic non-capturing group, e.g. `(?>ab|a)` in text that contains `ab` will match `ab` and
     /// never backtrack and try `a`, even if matching fails after the atomic group.
     AtomicGroup(Box<Expr>),
+    /// Keep matched text so far out of overall match
+    KeepOut,
+    /// Anchor to match at the position where the previous match ended
+    ContinueFromPreviousMatchEnd,
+    /// Conditional expression based on whether the numbered capture group matched or not
+    BackrefExistsCondition(usize),
+    /// If/Then/Else Condition. If there is no Then/Else, these will just be empty expressions.
+    Conditional {
+        /// The conditional expression to evaluate
+        condition: Box<Expr>,
+        /// What to execute if the condition is true
+        true_branch: Box<Expr>,
+        /// What to execute if the condition is false
+        false_branch: Box<Expr>,
+    },
 }
 
 /// Type of look-around assertion as used for a look-around expression.
@@ -1308,27 +1358,12 @@ impl Expr {
                 if precedence > 0 {
                     buf.push_str("(?:");
                 }
-
-                let is_empty = |e: &Expr| match e {
-                    Expr::Empty => true,
-                    _ => false,
-                };
-
-                let contains_empty = children.iter().any(is_empty);
-                if contains_empty {
-                    buf.push_str("(?:");
-                }
-                for (i, child) in children.iter().filter(|&c| !is_empty(c)).enumerate() {
+                for (i, child) in children.iter().enumerate() {
                     if i != 0 {
                         buf.push('|');
                     }
                     child.to_str(buf, 1);
                 }
-                if contains_empty {
-                    // regex fails with `(a|b|)`, so transform to `((?:a|b)?)`
-                    buf.push_str(")?");
-                }
-
                 if precedence > 0 {
                     buf.push(')');
                 }
