@@ -20,7 +20,7 @@
 
 //! Backtracking VM for implementing fancy regexes.
 //!
-//! Read https://swtch.com/~rsc/regexp/regexp2.html for a good introduction for how this works.
+//! Read <https://swtch.com/~rsc/regexp/regexp2.html> for a good introduction for how this works.
 //!
 //! The VM executes a sequence of instructions (a program) against an input string. It keeps track
 //! of a program counter (PC) and an index into the string (IX). Execution can have one or more
@@ -73,12 +73,21 @@ use regex::Regex;
 use std::collections::BTreeSet;
 use std::usize;
 
+use crate::error::RuntimeError;
 use crate::prev_codepoint_ix;
 use crate::Error;
 use crate::Result;
 use crate::{codepoint_len, RegexOptions};
 
-const OPTION_TRACE: u32 = 1;
+/// Enable tracing of VM execution. Only for debugging/investigating.
+const OPTION_TRACE: u32 = 1 << 0;
+/// When iterating over all matches within a text (e.g. with `find_iter`), empty matches need to be
+/// handled specially. If we kept matching at the same position, we'd never stop. So what we do
+/// after we've had an empty match, is to advance the position where matching is attempted.
+/// If `\G` is used in the pattern, that means it no longer matches. If we didn't tell the VM about
+/// the fact that we skipped because of an empty match, it would still treat `\G` as matching. So
+/// this option is for communicating that to the VM. Phew.
+pub(crate) const OPTION_SKIPPED_EMPTY_MATCH: u32 = 1 << 1;
 
 // TODO: make configurable
 const MAX_STACK: usize = 1_000_000;
@@ -181,6 +190,10 @@ pub enum Insn {
         /// The last group number
         end_group: usize,
     },
+    /// Anchor to match at the position where the previous match ended
+    ContinueFromPreviousMatchEnd,
+    /// Continue only if the specified capture group has already been populated as part of the match
+    BackrefExistsCondition(usize),
 }
 
 /// Sequence of instructions for the VM to execute.
@@ -264,7 +277,7 @@ impl State {
             self.trace_stack("push");
             Ok(())
         } else {
-            Err(Error::StackOverflow)
+            Err(Error::RuntimeError(RuntimeError::StackOverflow))
         }
     }
 
@@ -439,6 +452,13 @@ pub(crate) fn run(
                     if option_flags & OPTION_TRACE != 0 {
                         println!("saves: {:?}", state.saves);
                     }
+                    if let Some(&slot1) = state.saves.get(1) {
+                        // With some features like keep out (\K), the match start can be after
+                        // the match end. Cap the start to <= end.
+                        if state.get(0) > slot1 {
+                            state.save(0, slot1);
+                        }
+                    }
                     return Ok(Some(state.saves));
                 }
                 Insn::Any => {
@@ -577,6 +597,10 @@ pub(crate) fn run(
                         break 'fail;
                     }
                     let hi = state.get(slot + 1);
+                    if hi == usize::MAX {
+                        // Referenced group hasn't matched, so the backref doesn't match either
+                        break 'fail;
+                    }
                     let ref_text = &s[lo..hi];
                     let ix_end = ix + ref_text.len();
                     if !matches_literal(s, ix, ix_end, ref_text) {
@@ -584,6 +608,13 @@ pub(crate) fn run(
                     }
                     ix = ix_end;
                 }
+                Insn::BackrefExistsCondition(group) => {
+                    let lo = state.get(group * 2);
+                    if lo == usize::MAX {
+                        // Referenced group hasn't matched, so the backref doesn't match either
+                        break 'fail;
+                    }
+                }
                 Insn::BeginAtomic => {
                     let count = state.backtrack_count();
                     state.stack_push(count);
@@ -646,6 +677,11 @@ pub(crate) fn run(
                         }
                     }
                 }
+                Insn::ContinueFromPreviousMatchEnd => {
+                    if ix > pos || option_flags & OPTION_SKIPPED_EMPTY_MATCH != 0 {
+                        break 'fail;
+                    }
+                }
             }
             pc += 1;
         }
@@ -659,7 +695,7 @@ pub(crate) fn run(
 
         backtrack_count += 1;
         if backtrack_count > options.backtrack_limit {
-            return Err(Error::BacktrackLimitExceeded);
+            return Err(Error::RuntimeError(RuntimeError::BacktrackLimitExceeded));
         }
 
         let (newpc, newix) = state.pop();
